@@ -1,9 +1,9 @@
 package buoy.widget;
 
-import buoy.event.*;
-import buoy.internal.*;
 import buoy.xml.*;
 import buoy.xml.delegate.*;
+import buoy.event.*;
+
 import java.awt.*;
 import java.awt.event.*;
 import java.util.*;
@@ -42,10 +42,14 @@ public class BScrollPane extends WidgetContainer
   public static final ScrollbarPolicy SCROLLBAR_NEVER = new ScrollbarPolicy();
   public static final ScrollbarPolicy SCROLLBAR_AS_NEEDED = new ScrollbarPolicy();
   public static final ScrollbarPolicy SCROLLBAR_ALWAYS = new ScrollbarPolicy();
-  
+
+  private static final boolean IS_MACINTOSH;
+
   static
   {
     WidgetEncoder.setPersistenceDelegate(ScrollbarPolicy.class, new StaticFieldDelegate(BScrollPane.class));
+    String os = System.getProperty("os.name", "").toLowerCase();
+    IS_MACINTOSH = os.startsWith("mac os x");
   }
   
   /**
@@ -82,15 +86,10 @@ public class BScrollPane extends WidgetContainer
   
   public BScrollPane(ScrollbarPolicy horizontalPolicy, ScrollbarPolicy verticalPolicy)
   {
-    WidgetContainerPanel panel = new WidgetContainerPanel(this);
+    JScrollPane panel = createComponent();
     component = panel;
     hPolicy = horizontalPolicy;
     vPolicy = verticalPolicy;
-    panel.add(contentPort = new ContentViewport());
-    panel.add(rowHeaderPort = new JViewport());
-    panel.add(colHeaderPort = new JViewport());
-    forceWidth = (hPolicy == SCROLLBAR_NEVER);
-    forceHeight = (vPolicy == SCROLLBAR_NEVER);
     ChangeListener scrollListener = new ChangeListener() {
       public void stateChanged(ChangeEvent e)
       {
@@ -125,7 +124,6 @@ public class BScrollPane extends WidgetContainer
           bar.setValue(bar.getValue()+bar.getUnitIncrement(direction)*ev.getScrollAmount()*direction);
       }
     };
-    contentPort.addMouseWheelListener(wheelListener);
     vscroll = new ScrollPaneScrollBar(0, 1, 0, 100, BScrollBar.VERTICAL);
     panel.add(vscroll.component);
     setAsParent(vscroll);
@@ -136,6 +134,16 @@ public class BScrollPane extends WidgetContainer
     setAsParent(hscroll);
     ((JScrollBar) hscroll.component).getModel().addChangeListener(scrollListener);
     hscroll.component.addMouseWheelListener(wheelListener);
+    panel.setViewport(contentPort = new ContentViewport());
+    panel.setRowHeader(rowHeaderPort = new JViewport());
+    panel.setColumnHeader(colHeaderPort = new JViewport());
+    rowHeaderPort.setLayout(null);
+    colHeaderPort.setLayout(null);
+    rowHeaderPort.setOpaque(false);
+    colHeaderPort.setOpaque(false);
+    forceWidth = (hPolicy == SCROLLBAR_NEVER);
+    forceHeight = (vPolicy == SCROLLBAR_NEVER);
+    contentPort.addMouseWheelListener(wheelListener);
   }
   
   /**
@@ -154,6 +162,16 @@ public class BScrollPane extends WidgetContainer
     setContent(contentWidget);
   }
 
+  /**
+   * Create the JScrollPane which serves as this Widget's Component.  This method is protected so that
+   * subclasses can override it.
+   */
+
+  protected JScrollPane createComponent()
+  {
+    return new ScrollPaneComponent();
+  }
+
   /**
    * Get the content Widget.
    */
@@ -395,10 +413,10 @@ public class BScrollPane extends WidgetContainer
   }
   
   /**
-   * Get an Iterator listing all child Widgets.
+   * Get a Collection containing all child Widgets of this container.
    */
   
-  public Iterator getChildren()
+  public Collection getChildren()
   {
     ArrayList ls = new ArrayList(5);
     if (content != null)
@@ -409,7 +427,7 @@ public class BScrollPane extends WidgetContainer
       ls.add(colHeader);
     ls.add(hscroll);
     ls.add(vscroll);
-    return ls.iterator();
+    return ls;
   }
   
   /**
@@ -522,10 +540,36 @@ public class BScrollPane extends WidgetContainer
         size.height = forceSize.height;
       content.component.setSize(size);
     }
+
+    // On a Macintosh, if the scroll pane is in the lower right corner of a resizable window,
+    // leave room for the grow box.
+
+    int hScrollLength = viewBounds.width;
+    int vScrollLength = viewBounds.height;
+    if (IS_MACINTOSH && hasHScroll != hasVScroll)
+    {
+      Widget parent = getParent();
+      while (parent != null && !(parent instanceof WindowWidget))
+        parent = parent.getParent();
+      if ((parent instanceof BFrame && ((BFrame) parent).isResizable()) ||
+          (parent instanceof BDialog && ((BDialog) parent).isResizable()))
+      {
+        Rectangle parentBounds = parent.getBounds();
+        Point offset = SwingUtilities.convertPoint(getComponent(), 0, 0, parent.getComponent());
+        if (bounds.x+bounds.width+offset.x == parentBounds.width &&
+            bounds.y+bounds.height+offset.y == parentBounds.height)
+        {
+          if (!hasHScroll)
+            vScrollLength -= hScrollSize.height;
+          if (!hasVScroll)
+            hScrollLength -= vScrollSize.width;
+        }
+      }
+    }
     
     // Set up the scrollbars.
     
-    hscroll.component.setBounds(new Rectangle(leftMargin, viewBounds.y+viewBounds.height, viewBounds.width, bottomMargin));
+    hscroll.component.setBounds(new Rectangle(leftMargin, viewBounds.y+viewBounds.height, hScrollLength, bottomMargin));
     if (content == null)
       hscroll.setEnabled(false);
     else
@@ -537,7 +581,7 @@ public class BScrollPane extends WidgetContainer
         hscroll.setValue(width-viewBounds.width);
     }
     hscroll.setExtent(viewBounds.width);
-    vscroll.component.setBounds(new Rectangle(viewBounds.x+viewBounds.width, topMargin, rightMargin, viewBounds.height));
+    vscroll.component.setBounds(new Rectangle(viewBounds.x+viewBounds.width, topMargin, rightMargin, vScrollLength));
     if (content == null)
       vscroll.setEnabled(false);
     else
@@ -677,6 +721,36 @@ public class BScrollPane extends WidgetContainer
     prefSize = minSize = null;
     super.invalidateSize();
   }
+
+  /**
+   * This is a JScrollPane subclass which is the component for the BScrollPane.  It is nearly identical
+   * to WidgetContainerPanel, except that it extends JScrollPane instead of JPanel.
+   */
+
+  private class ScrollPaneComponent extends JScrollPane
+  {
+    private ScrollPaneComponent()
+    {
+      setLayout(null);
+    }
+
+    public void paintComponent(Graphics g)
+    {
+      if (BScrollPane.this.isOpaque())
+      {
+        Dimension size = getSize();
+        g.setColor(getBackground());
+        g.fillRect(0, 0, size.width, size.height);
+        g.setColor(getForeground());
+      }
+      BScrollPane.this.dispatchEvent(new RepaintEvent(BScrollPane.this, (Graphics2D) g));
+    }
+
+    public boolean isOpaque()
+    {
+      return BScrollPane.this.isOpaque();
+    }
+  }
   
   /**
    * This is a BScrollBar subclass which is used for the scrollbars of the BScrollPane.  If the content
